# -*- coding: utf-8 -*-
from __future__ import print_function

from blessings import Terminal
import readchar

from inquirer import errors
from .text import Text
from .password import Password
from .confirm import Confirm
from .list import List
from .checkbox import Checkbox


class ConsoleRender(object):
    def __init__(self, key_generator=None, *args, **kwargs):
        super(ConsoleRender, self).__init__(*args, **kwargs)
        self._key_gen = key_generator or readchar.readkey
        self.terminal = Terminal()

    def render(self, question, answers=None):
        question.answers = answers or {}

        if question.ignore:
            return question.default

        clazz = self.render_factory(question.kind)
        render = clazz(self._key_gen, self.terminal)

        render.clear_eos()

        while True:
            try:
                result = render.render(question)
                question.validate(result)
                return result
            except errors.ValidationError:
                msg = ('Invalid value for {q}.'.format(q=question.name))
                render.render_error(msg)

    def render_factory(self, question_type):
        matrix = {
            'text': Text,
            'password': Password,
            'confirm': Confirm,
            'list': List,
            'checkbox': Checkbox,
            }

        if question_type not in matrix:
            raise errors.UnknownQuestionTypeError()
        return matrix.get(question_type)



import unittest
import doublex

from inquirer import prompt


class PromptTests(unittest.TestCase):
    def test_prompt_returns_a_hash(self):
        self.assertEquals({}, prompt([]))

    def test_prompt_renders_a_questions(self):
        question1 = doublex.Stub()
        question1.name = 'foo'
        result1 = object()
        with doublex.Mock() as render:
            render.render(question1, {}).returns(result1)

        result = prompt([question1], render=render)

        self.assertEquals({'foo': result1}, result)

    def test_prompt_renders_all_questions(self):
        question1 = doublex.Stub()
        question1.name = 'foo'
        result1 = object()

        question2 = doublex.Stub()
        question2.name = 'bar'
        result2 = object()

        result = object()
        with doublex.Mock() as render:
            render.render(question1, {}).returns(result1)
            render.render(question2, {'foo': result1}).returns(result2)

        result = prompt([question1, question2], render=render)

        self.assertEquals({'foo': result1, 'bar': result2}, result)



import unittest
import inquirer.questions as questions

from . import helper
from readchar import key

from inquirer.render import ConsoleRender


class ListRenderTest(unittest.TestCase, helper.BaseTestCase):
    def setUp(self):
        self.base_setup()

    def tearDown(self):
        self.base_teardown()

    def test_all_choices_are_shown(self):
        stdin = helper.key_factory(key.ENTER)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.List(variable, message, choices=choices)

        sut = ConsoleRender(key_generator=stdin)
        sut.render(question)

        self.assertInStdout(message)
        for choice in choices:
            self.assertInStdout(choice)

    def test_choose_the_first(self):
        stdin = helper.key_factory(key.ENTER)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.List(variable, message, choices=choices)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEqual('foo', result)

    def test_choose_the_second(self):
        stdin = helper.key_factory(key.DOWN, key.ENTER)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.List(variable, message, choices=choices)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEqual('bar', result)

    def test_move_up(self):
        stdin = helper.key_factory(key.DOWN, key.UP, key.ENTER)
        message = 'Foo message'
        variable = 'Bar variable'
        choices = ['foo', 'bar', 'bazz']

        question = questions.List(variable, message, choices=choices)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEqual('foo', result)

    def test_ctrl_c_breaks_execution(self):
        stdin_array = [key.CTRL_C]
        stdin = helper.key_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.List(variable, message)

        sut = ConsoleRender(key_generator=stdin)
        with self.assertRaises(KeyboardInterrupt):
            sut.render(question)



# -*- coding: utf-8 -*-

from .render.console import ConsoleRender


def prompt(questions, render=None, answers=None):
    render = render or ConsoleRender()
    answers = answers or {}

    for question in questions:
        answers[question.name] = render.render(question, answers)
    return answers



import unittest
import pexpect
from readchar import key


class PasswordTest(unittest.TestCase):
    def setUp(self):
        self.sut = pexpect.spawn('python examples/password.py')

    def test_default_input(self):
        self.sut.expect("What's your password.*", timeout=1)
        self.sut.send('abcde')
        self.sut.send(key.ENTER)
        self.sut.expect("{'password': 'abcde'}", timeout=1)

    def test_backspace(self):
        self.sut.expect("What's your password.*", timeout=1)
        self.sut.send('abcde')
        self.sut.send(key.BACKSPACE)
        self.sut.send(key.ENTER)
        self.sut.expect("{'password': 'abcd'}", timeout=1)

    def test_backspace_limit(self):
        self.sut.expect("What's your password.*", timeout=1)
        self.sut.send('a')
        self.sut.send(key.BACKSPACE)
        self.sut.send(key.BACKSPACE)
        self.sut.send('b')
        self.sut.send(key.ENTER)
        self.sut.expect("{'password': 'b'}", timeout=1)



# -*- coding: utf-8 -*-

from __future__ import print_function

import sys
from blessings import Terminal
import readchar
# from inquirer import errors


class ConsoleRender(object):
    def __init__(self, key_generator=None, terminal=None, *args, **kwargs):
        super(ConsoleRender, self).__init__(*args, **kwargs)
        self._key_gen = key_generator or readchar.readkey
        self.terminal = terminal or Terminal()
        self.answers = {}

    def render_error(self, message):
        if message:
            self.render_in_bottombar(
                '{t.red}>> {t.normal}{t.bold}{msg}{t.normal} '
                .format(msg=message, t=self.terminal)
                )

    def render_in_bottombar(self, message):
        with self.terminal.location(0, self.terminal.height - 1):
            self.terminal.clear_eos()
            self.print_str(message)

    def print_line(self, base, **kwargs):
        self.print_str(base + self.terminal.clear_eol(), lf=True, **kwargs)

    def print_str(self, base, lf=False, **kwargs):
        print(base.format(t=self.terminal, **kwargs), end='\n' if lf else '')
        sys.stdout.flush()

    def clear_eos(self, lf=True):
        print(self.terminal.clear_eos(), end='\n' if lf else '')



import re
import unittest
import pexpect


class TextTest(unittest.TestCase):
    def setUp(self):
        self.sut = pexpect.spawn('python examples/text.py')

    def set_name(self, name='foo'):
        self.sut.expect("What's your name.*", timeout=1)
        self.sut.sendline(name)

    def set_surname(self, surname='bar'):
        self.sut.expect("What's your surname.*", timeout=1)
        self.sut.sendline(surname)

    def set_phone(self, phone='123456789'):
        self.sut.expect("What's your phone.*", timeout=1)
        self.sut.sendline(phone)

    def test_default_input(self):
        self.set_name()
        self.set_surname()
        self.set_phone()
        self.sut.expect_list([re.compile(b"'name': 'foo'"),
                              re.compile(b"'surname': 'bar'"),
                              re.compile(b"'phone': '123456789'")],
                             timeout=1)

    def test_invalid_phone(self):
        self.set_name()
        self.set_surname()
        self.set_phone('abcde')
        self.sut.expect('Invalid value.*', timeout=1)
        self.sut.sendline('12345')
        self.sut.expect_list([re.compile(b"'name': 'foo'"),
                              re.compile(b"'surname': 'bar'"),
                              re.compile(b"'phone': '12345'")],
                             timeout=1)



import unittest
import pexpect
from readchar import key


class ListTest(unittest.TestCase):
    def setUp(self):
        self.sut = pexpect.spawn('python examples/checkbox.py')
        self.sut.expect('History.*', timeout=1)

    def test_default_input(self):
        self.sut.send(key.ENTER)
        self.sut.expect("{'interests': \[\]}.*", timeout=1)

    def test_select_the_first(self):
        self.sut.send(key.SPACE)
        self.sut.send(key.ENTER)
        self.sut.expect("{'interests': \['Computers'\]}.*", timeout=1)

    def test_select_two(self):
        self.sut.send(key.SPACE)
        self.sut.send(key.DOWN)
        self.sut.send(key.SPACE)
        self.sut.send(key.ENTER)
        self.sut.expect("{'interests': \['Computers', 'Books'\]}.*", timeout=1)

    def test_unselect(self):
        self.sut.send(key.SPACE)
        self.sut.send(key.SPACE)
        self.sut.send(key.ENTER)
        self.sut.expect("{'interests': \[\]}.*", timeout=1)

    def test_select_with_arrows(self):
        self.sut.send(key.RIGHT)
        self.sut.send(key.ENTER)
        self.sut.expect("{'interests': \['Computers'\]}.*", timeout=1)

    def test_unselect_with_arrows(self):
        self.sut.send(key.SPACE)
        self.sut.send(key.LEFT)
        self.sut.send(key.ENTER)
        self.sut.expect("{'interests': \[\]}.*", timeout=1)

    def test_select_last(self):
        for i in range(10):
            self.sut.send(key.DOWN)
        self.sut.send(key.SPACE)
        self.sut.send(key.ENTER)
        self.sut.expect("{'interests': \['History'\]}.*", timeout=1)



# -*- coding: utf-8 -*-

from readchar import key
from .base import ConsoleRender


class Text(ConsoleRender):

    def render(self, question):
        with self.terminal.location(0, self.terminal.height - 2):
            text = question.default or ''
            message = ('[{t.yellow}?{t.normal}] {msg}: {default}'
                       .format(msg=question.message,
                               t=self.terminal,
                               default=text))
            self.print_str(message)
            while True:
                pressed = self._key_gen()
                if pressed == key.CTRL_C:
                    raise KeyboardInterrupt()
                if pressed in (key.CR, key.LF, key.ENTER):
                    break
                if len(pressed) != 1:
                    continue
                if pressed == key.BACKSPACE:
                    if len(text):
                        text = text[:-1]
                        self.print_str(self.terminal.move_left
                                       + self.terminal.clear_eol)
                else:
                    text += pressed
                    self.print_str(pressed)

            return text



# -*- coding: utf-8 -*-

from . import errors


class Question(object):
    kind = 'base question'

    def __init__(self,
                 name,
                 message='',
                 choices=None,
                 default=None,
                 ignore=False,
                 validate=True):
        self.name = name
        self._message = message
        self._choices = choices or []
        self._default = default
        self._ignore = ignore
        self._validate = validate
        self.answers = {}

    @property
    def ignore(self):
        return bool(self._solve(self._ignore))

    @property
    def message(self):
        return self._solve(self._message)

    @property
    def default(self):
        return self._solve(self._default)

    @property
    def choices(self):
        return self._solve(self._choices)

    def validate(self, current):
        try:
            if self._solve(self._validate, current):
                return
        except Exception:
            pass
        raise errors.ValidationError()

    def _solve(self, prop, *args, **kwargs):
        if callable(prop):
            return prop(self.answers, *args, **kwargs)
        if isinstance(prop, str):
            return prop.format(**self.answers)
        return prop


class Text(Question):
    kind = 'text'


class Password(Question):
    kind = 'password'


class Confirm(Question):
    kind = 'confirm'

    def __init__(self, name, default=False, **kwargs):
        super(Confirm, self).__init__(name, default=default, **kwargs)


class List(Question):
    kind = 'list'


class Checkbox(Question):
    kind = 'checkbox'



import re
import unittest
import inquirer.questions as questions

from . import helper
from readchar import key

from inquirer.render import ConsoleRender


class TextRenderTest(unittest.TestCase, helper.BaseTestCase):
    def setUp(self):
        self.base_setup()

    def tearDown(self):
        self.base_teardown()

    def test_basic_render(self):
        stdin_msg = 'This is a foo message'
        stdin_array = [x for x in stdin_msg + key.ENTER]
        stdin = helper.key_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Text(variable, message)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEquals(stdin_msg, result)
        self.assertInStdout(message)

    def test_ignore_true_should_return(self):
        stdin = 'This is a foo message'
        message = 'Foo message'
        variable = 'Bar variable'
        expected = object()

        question = questions.Text(variable,
                                  ignore=True,
                                  default=expected,
                                  message=message)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEquals(expected, result)
        self.assertNotInStdout(message)

    def test_validation_fails(self):
        stdin_array = [x for x in
                       'Invalid' + key.ENTER
                       + '9999' + key.ENTER]
        stdin = helper.key_factory(*stdin_array)
        message = 'Insert number'
        variable = 'foo'
        expected = '9999'

        question = questions.Text(variable,
                                  validate=lambda _, x: re.match('\d+', x),
                                  message=message)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)
        self.assertEquals(expected, result)
        self.assertInStdout(message)
        self.assertInStdout('Invalid value')

    def test_allows_deletion(self):
        stdin_array = ['a', key.BACKSPACE, 'b', key.ENTER]
        stdin = helper.key_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Text(variable, message)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEquals('b', result)

    def test_ignore_cursors(self):
        stdin_array = [
            'a',
            key.UP,
            'b',
            key.DOWN,
            'c',
            key.LEFT,
            'd',
            key.RIGHT,
            'e',
            key.ENTER,
            ]
        stdin = helper.key_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Text(variable, message)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEquals('abcde', result)

    def test_ctrl_c_breaks_execution(self):
        stdin_array = [key.CTRL_C]
        stdin = helper.key_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Text(variable, message)

        sut = ConsoleRender(key_generator=stdin)
        with self.assertRaises(KeyboardInterrupt):
            sut.render(question)



from __future__ import print_function

__version__ = '1.0.3'

try:
    from .prompt import prompt
    from .questions import Text, Password, Confirm, List, Checkbox

    __all__ = ['prompt', 'Text', 'Password', 'Confirm', 'List', 'Checkbox']
except ImportError as e:
    print("An error was found, but returning just with the version: %s" % e)



import sys

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


def key_factory(*args):
    iterator = args.__iter__()

    def inner():
        return next(iterator)
    return inner


class BaseTestCase(object):
    def base_setup(self):
        self._base_stdin = sys.stdin
        self._base_stdout = sys.stdout
        sys.stdin = StringIO()
        sys.stdout = StringIO()

    def base_teardown(self):
        sys.stdin = self._base_stdin
        sys.stdout = self._base_stdout

    def printStdout(self):
        sys.stdout.seek(0)
        self._base_stdout.write(sys.stdout.read())

    def assertInStdout(self, message):
        sys.stdout.seek(0)
        stdout = sys.stdout.read()
        self.assertIn(message, stdout)

    def assertNotInStdout(self, message):
        sys.stdout.seek(0)
        stdout = sys.stdout.read()
        self.assertNotIn(message, stdout)



# -*- coding: utf-8 -*-

from readchar import key
from .base import ConsoleRender


class List(ConsoleRender):

    def render(self, question):
        choices = question.choices
        try:
            current = choices.index(question.default)
        except ValueError:
            current = 0

        self.print_line('[{t.yellow}?{t.normal}] {msg}: ',
                        msg=question.message)
        for choice in choices:
            print('')
        print(self.terminal.clear_eos())

        pos_y = self.terminal.height - 2 - len(choices)

        while True:
            with self.terminal.location(0, pos_y):
                for choice in choices:
                    if choice == choices[current]:
                        color = self.terminal.blue
                        symbol = '>'
                    else:
                        color = self.terminal.normal
                        symbol = ' '
                    self.print_line(' {color}{s} {c}{t.normal}',
                                    c=choice, color=color, s=symbol)
                pressed = self._key_gen()
                if pressed == key.UP:
                    current = max(0, current - 1)
                    continue
                if pressed == key.DOWN:
                    current = min(len(choices) - 1, current + 1)
                    continue
                if pressed == key.ENTER:
                    return choices[current]
                if pressed == key.CTRL_C:
                    raise KeyboardInterrupt()



import unittest
import doublex

from inquirer.render import Render


class RenderTests(unittest.TestCase):
    def test_calls_the_implementation(self):
        question = object()
        answers = object()
        with doublex.Mock() as render_impl:
            render_impl.render(question, answers)
        sut = Render(render_impl)

        sut.render(question, answers)



# -*- coding: utf-8 -*-
from .console import ConsoleRender


class Render(object):
    def __init__(self, impl=ConsoleRender):
        self._impl = impl

    def render(self, question, answers):
        return self._impl.render(question, answers)



import unittest
import inquirer.questions as questions
import inquirer.errors as errors

from . import helper

from inquirer.render import ConsoleRender


class BasicTest(unittest.TestCase, helper.BaseTestCase):
    def test_rendering_erroneous_type(self):
        question = questions.Question('foo', 'bar')

        sut = ConsoleRender()
        with self.assertRaises(errors.UnknownQuestionTypeError):
            sut.render(question)



# -*- coding: utf-8 -*-

from readchar import key
from .base import ConsoleRender


class Checkbox(ConsoleRender):

    def render(self, question):
        choices = question.choices
        selection = []
        current = 0

        self.print_line('[{t.yellow}?{t.normal}] {msg}: ',
                        msg=question.message)
        for choice in choices:
            self.print_line('')
        self.clear_eos()

        pos_y = self.terminal.height - 2 - len(choices)

        while True:
            with self.terminal.location(0, pos_y):
                for n in range(len(choices)):
                    choice = choices[n]
                    if n in selection:
                        symbol = 'X'
                        color = self.terminal.yellow + self.terminal.bold
                    else:
                        symbol = 'o'
                        color = ''
                    selector = ' '
                    if n == current:
                        selector = '>'
                        color = self.terminal.blue
                    self.print_line(' {color}{sel} {s} {c}{t.normal}',
                                    c=choice, s=symbol, sel=selector,
                                    color=color)
                pressed = self._key_gen()
                if pressed == key.UP:
                    current = max(0, current - 1)
                    continue
                elif pressed == key.DOWN:
                    current = min(len(choices) - 1, current + 1)
                    continue
                elif pressed == key.SPACE:
                    if current in selection:
                        selection.remove(current)
                    else:
                        selection.append(current)
                elif pressed == key.LEFT:
                    if current in selection:
                        selection.remove(current)
                elif pressed == key.RIGHT:
                    if current not in selection:
                        selection.append(current)
                elif pressed == key.ENTER:
                    return [choices[x] for x in selection]
                elif pressed == key.CTRL_C:
                    raise KeyboardInterrupt()



# -*- coding: utf-8 -*-

from readchar import key

from .base import ConsoleRender


class Password(ConsoleRender):

    def render(self, question):
        with self.terminal.location(0, self.terminal.height - 2):
            self.terminal.clear_eos(False)
            self.print_str('[{t.yellow}?{t.normal}] {msg}: ',
                           msg=question.message)
            password = ''
            while True:
                pressed = self._key_gen()
                if pressed == key.CTRL_C:
                    raise KeyboardInterrupt()
                if pressed == key.ENTER:
                    break
                if len(pressed) != 1:
                    continue
                if pressed == key.BACKSPACE:
                    if len(password):
                        password = password[:-1]
                        self.print_str(self.terminal.move_left
                                       + self.terminal.clear_eol)
                else:
                    password += pressed
                    self.print_str('*')
            return password



import unittest
import inquirer.questions as questions

from . import helper
from readchar import key

from inquirer.render import ConsoleRender


class PasswordRenderTest(unittest.TestCase, helper.BaseTestCase):
    def setUp(self):
        self.base_setup()

    def tearDown(self):
        self.base_teardown()

    def test_do_not_show_values(self):
        stdin = helper.key_factory(
            'm', 'y', ' ', 'p', 'a', 's', 's', 'w', 'o', 'r', 'd',
            key.ENTER)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Password(variable, message)

        sut = ConsoleRender(key_generator=stdin)
        sut.render(question)

        self.assertInStdout(message)
        self.assertNotInStdout('my password')

    def test_allows_deletion(self):
        stdin_array = ['a', key.BACKSPACE, 'b', key.ENTER]
        stdin = helper.key_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Password(variable, message)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEquals('b', result)

    def test_ignore_cursors(self):
        stdin_array = [
            'a',
            key.UP,
            'b',
            key.DOWN,
            'c',
            key.LEFT,
            'd',
            key.RIGHT,
            'e',
            key.ENTER,
            ]
        stdin = helper.key_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Password(variable, message)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEquals('abcde', result)

    def test_ctrl_c_breaks_execution(self):
        stdin_array = [key.CTRL_C]
        stdin = helper.key_factory(*stdin_array)
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Password(variable, message)

        sut = ConsoleRender(key_generator=stdin)
        with self.assertRaises(KeyboardInterrupt):
            sut.render(question)



import unittest
import pexpect
from readchar import key


class ListTest(unittest.TestCase):
    def setUp(self):
        self.sut = pexpect.spawn('python examples/list.py')
        self.sut.expect('Micro.*', timeout=1)

    def test_default_input(self):
        self.sut.send(key.ENTER)
        self.sut.expect("{'size': 'Jumbo'}.*", timeout=1)

    def test_change_selection(self):
        self.sut.send(key.DOWN)
        self.sut.expect('Micro.*', timeout=1)
        self.sut.send(key.ENTER)
        self.sut.expect("{'size': 'Large'}.*", timeout=1)

    def test_out_of_bounds_up(self):
        self.sut.send(key.UP)
        self.sut.expect('Micro.*', timeout=1)
        self.sut.send(key.ENTER)
        self.sut.expect("{'size': 'Jumbo'}.*", timeout=1)

    def test_out_of_bounds_down(self):
        for i in range(10):
            self.sut.send(key.DOWN)
            self.sut.expect('Micro.*', timeout=1)
        self.sut.send(key.ENTER)
        self.sut.expect("{'size': 'Micro'}.*", timeout=1)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from inquirer import __version__


def read_description():
    with open('README.rst') as fd:
        return fd.read()


setup(name='inquirer',
      version=__version__,
      description="Collection of common interactive command line user interfaces, based on Inquirer.js",
      long_description=read_description(),
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent',
          'Topic :: Software Development :: User Interfaces',
          'Topic :: Software Development :: Libraries :: Application Frameworks',
      ],
      keywords='color',
      author='Miguel Ángel García',
      author_email='miguelangel.garcia@gmail.com',
      url='https://github.com/magmax/python-inquirer',
      license='MIT',
      packages=find_packages(exclude=['tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'blessings >= 1.5.1',
          'readchar >= 0.7',
      ],
      )



import unittest

from inquirer import questions
from inquirer import errors


class BaseQuestionTests(unittest.TestCase):
    def test_base_question_type(self):
        name = 'foo'
        q = questions.Question(name)

        self.assertEquals('base question', q.kind)
        self.assertEquals(name, q.name)

    def test_ignore_works_for_true(self):
        name = 'foo'
        q = questions.Question(name, ignore=True)

        self.assertTrue(q.ignore)

    def test_ignore_works_for_false(self):
        name = 'foo'
        q = questions.Question(name, ignore=False)

        self.assertFalse(q.ignore)

    def test_ignore_works_for_function_returning_true(self):
        name = 'foo'
        q = questions.Question(name, ignore=lambda x: True)

        self.assertTrue(q.ignore)

    def test_ignore_works_for_function_returning_false(self):
        name = 'foo'
        q = questions.Question(name, ignore=lambda x: False)

        self.assertFalse(q.ignore)

    def test_ignore_works_for_function_returning_none(self):
        name = 'foo'
        q = questions.Question(name, ignore=lambda x: None)

        self.assertFalse(q.ignore)

    def test_ignore_function_receives_answers(self):
        name = 'foo'
        method = lambda x: isinstance(x, dict)
        q = questions.Question(name, ignore=method)

        self.assertTrue(q.ignore, "Method was not called with a dict instance")

    def test_default_message_is_empty(self):
        name = 'foo'
        q = questions.Question(name)

        self.assertEqual('', q.message)

    def test_message_set(self):
        name = 'foo'
        message = 'bar'
        q = questions.Question(name, message=message)

        self.assertEqual(message, q.message)

    def test_message_previous_answers_replacement(self):
        name = 'foo'
        message = 'replacement == {whatever}'
        expected = 'replacement == replacement'
        q = questions.Question(name, message=message)
        q.answers = {'whatever': 'replacement'}

        self.assertEqual(expected, q.message)

    def test_default_default_value(self):
        name = 'foo'
        q = questions.Question(name)

        self.assertIsNone(q.default)

    def test_setting_default_value(self):
        name = 'foo'
        expected = object()
        q = questions.Question(name, default=expected)

        self.assertEqual(expected, q.default)

    def test_default_choices_value(self):
        name = 'foo'
        expected = []
        q = questions.Question(name)

        self.assertEqual(expected, q.choices)

    def test_setting_choices_value(self):
        name = 'foo'
        expected = (object(), )
        q = questions.Question(name, choices=expected)

        self.assertEqual(expected, q.choices)

    def test_validate_false_raises_exception(self):
        name = 'foo'
        q = questions.Question(name, validate=False)

        with self.assertRaises(errors.ValidationError):
            q.validate(None)

    def test_validate_function_returning_false_raises_exception(self):
        name = 'foo'
        q = questions.Question(name, validate=lambda x, y: False)

        with self.assertRaises(errors.ValidationError):
            q.validate(None)

    def test_validate_function_returning_true_ends_ok(self):
        name = 'foo'
        q = questions.Question(name, validate=lambda x, y: True)

        q.validate(None)

    def test_validate_function_raising_exception(self):
        def raise_exc(x, y):
            raise Exception('foo')
        name = 'foo'
        q = questions.Question(name, validate=raise_exc)

        with self.assertRaises(errors.ValidationError):
            q.validate(None)

    def test_validate_function_receives_object(self):
        expected = object()

        def compare(x, y):
            return expected == y
        name = 'foo'
        q = questions.Question(name, validate=compare)

        try:
            q.validate(expected)
        except errors.ValidationError:
            self.fail('Validation function did not receive the current value')


class TestConfirmQuestion(unittest.TestCase):
    def test_default_default_value_is_false_instead_of_none(self):
        name = 'foo'
        q = questions.Confirm(name)

        self.assertEquals(False, q.default)



import unittest
import inquirer.questions as questions

from . import helper
from readchar import key

from inquirer.render import ConsoleRender


class ConfirmRenderTest(unittest.TestCase, helper.BaseTestCase):
    def setUp(self):
        self.base_setup()

    def tearDown(self):
        self.base_teardown()

    def test_no_as_default(self):
        stdin = helper.key_factory(key.ENTER)
        message = 'Foo message'
        variable = 'Bar variable'
        expected = False

        question = questions.Confirm(variable,
                                     message=message)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEquals(expected, result)
        self.assertInStdout(message)
        self.assertInStdout('(y/N)')

    def test_yes_as_default(self):
        stdin = helper.key_factory(key.ENTER)
        message = 'Foo message'
        variable = 'Bar variable'
        expected = True

        question = questions.Confirm(variable,
                                     message=message,
                                     default=True)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEquals(expected, result)
        self.assertInStdout(message)
        self.assertInStdout('(Y/n)')

    def test_answring_y(self):
        stdin = helper.key_factory('y')
        message = 'Foo message'
        variable = 'Bar variable'
        expected = True

        question = questions.Confirm(variable,
                                     message=message,
                                     default=True)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEquals(expected, result)
        self.assertInStdout(message)
        self.assertInStdout('(Y/n)')

    def test_answring_Y(self):
        stdin = helper.key_factory('Y')
        message = 'Foo message'
        variable = 'Bar variable'
        expected = True

        question = questions.Confirm(variable,
                                     message=message,
                                     default=True)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEquals(expected, result)
        self.assertInStdout(message)
        self.assertInStdout('(Y/n)')

    def test_answring_n(self):
        stdin = helper.key_factory('n')
        message = 'Foo message'
        variable = 'Bar variable'
        expected = False

        question = questions.Confirm(variable,
                                     message=message,
                                     default=True)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEquals(expected, result)
        self.assertInStdout(message)
        self.assertInStdout('(Y/n)')

    def test_answring_N(self):
        stdin = helper.key_factory('N')
        message = 'Foo message'
        variable = 'Bar variable'
        expected = False

        question = questions.Confirm(variable,
                                     message=message,
                                     default=True)

        sut = ConsoleRender(key_generator=stdin)
        result = sut.render(question)

        self.assertEquals(expected, result)
        self.assertInStdout(message)
        self.assertInStdout('(Y/n)')

    def test_invalid_answer(self):
        stdin = helper.key_factory('a', 'N')
        message = 'Foo message'
        variable = 'Bar variable'

        question = questions.Confirm(variable,
                                     message=message,
                                     default=True)

        sut = ConsoleRender(key_generator=stdin)
        sut.render(question)

        self.assertInStdout('Invalid value')



# -*- coding: utf-8 -*-

from readchar import key
from inquirer import errors
from .base import ConsoleRender


class Confirm(ConsoleRender):

    def render(self, question):
        with self.terminal.location(0, self.terminal.height - 2):
            confirm = '(Y/n)' if question.default else '(y/N)'
            message = ('[{t.yellow}?{t.normal}] {msg} {c}: '
                       .format(msg=question.message,
                               t=self.terminal,
                               c=confirm))

            self.print_str(message)
            answer = self._key_gen()
            if answer.lower() == key.ENTER:
                return question.default
            self.print_str(answer)
            if answer in 'yY':
                return True
            if answer in 'nN':
                return False
            raise errors.ValidationError()
